@@ -60,7 +60,7 @@ class ReadFileJob(Job):
             begin_time = time.time()
             self.setResult(self._handler.readerRead(reader, self._filename))
             end_time = time.time()
-            Logger.log("d", "Loading file took %s seconds", end_time - begin_time)
+            Logger.log("d", "Loading file took %0.1f seconds", end_time - begin_time)
         except:
             Logger.logException("e", "Exception occurred while loading file %s", self._filename)
         finally:
@@ -69,4 +69,4 @@ class ReadFileJob(Job):
                 result_message = Message(i18n_catalog.i18nc("@info:status", "Failed to load <filename>{0}</filename>", self._filename), lifetime=0)
                 result_message.show()
                 return
-            self._loading_message.hide()
\ No newline at end of file
+            self._loading_message.hide()
