@@ -5,6 +5,8 @@ import json
 import collections
 import copy
 
+from PyQt5.QtCore import QObject, pyqtProperty
+
 from UM.Resources import Resources
 from UM.PluginObject import PluginObject
 from UM.Logger import Logger
@@ -18,12 +20,16 @@ from UM.Settings.SettingRelation import SettingRelation
 from UM.Settings.SettingRelation import RelationType
 from UM.Settings.SettingFunction import SettingFunction
 
+from typing import Dict, Any, List, Optional
+
 class InvalidDefinitionError(Exception):
     pass
 
+
 class IncorrectDefinitionVersionError(Exception):
     pass
 
+
 class InvalidOverrideError(Exception):
     pass
 
@@ -35,17 +41,20 @@ MimeTypeDatabase.addMimeType(
     )
 )
 
+
 ##  A container for SettingDefinition objects.
 #
 #
-class DefinitionContainer(DefinitionContainerInterface, PluginObject):
+class DefinitionContainer(QObject, DefinitionContainerInterface, PluginObject):
     Version = 2
 
     ##  Constructor
     #
     #   \param container_id A unique, machine readable/writable ID for this container.
-    def __init__(self, container_id, i18n_catalog = None, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, container_id: str, i18n_catalog = None, *args, **kwargs):
+        # Note that we explicitly pass None as QObject parent here. This is to be able
+        # to support pickling.
+        super().__init__(parent = None, *args, **kwargs)
 
         self._id = str(container_id)    # type: str
         self._name = container_id       # type: str
@@ -62,13 +71,28 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
         super().__setattr__(name, value)
         #raise NotImplementedError()
 
+    ##  For pickle support
+    def __getnewargs__(self):
+        return (self._id, self._i18n_catalog)
+
+    ##  For pickle support
+    def __getstate__(self):
+        return self.__dict__
+
+    ##  For pickle support
+    def __setstate__(self, state):
+        # We need to call QObject.__init__() in order to initialize the underlying C++ object.
+        # pickle doesn't do that so we have to do this here.
+        QObject.__init__(self, parent = None)
+        self.__dict__.update(state)
+
     ##  \copydoc ContainerInterface::getId
     #
     #   Reimplemented from ContainerInterface
     def getId(self) -> str:
         return self._id
 
-    id = property(getId)
+    id = pyqtProperty(str, fget = getId, constant = True)
 
     ##  \copydoc ContainerInterface::getName
     #
@@ -76,7 +100,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
     def getName(self) -> str:
         return self._name
 
-    name = property(getName)
+    name = pyqtProperty(str, fget = getName, constant = True)
 
     ##  \copydoc ContainerInterface::isReadOnly
     #
@@ -87,6 +111,8 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
     def setReadOnly(self, read_only: bool) -> None:
         pass
 
+    readOnly = pyqtProperty(bool, fget = isReadOnly, constant = True)
+
     ##  \copydoc ContainerInterface::getPath.
     #
     #   Reimplemented from ContainerInterface
@@ -105,7 +131,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
     def getMetaData(self):
         return self._metadata
 
-    metaData = property(getMetaData)
+    metaData = pyqtProperty("QVariantMap", fget = getMetaData, constant = True)
 
     @property
     def definitions(self):
@@ -126,7 +152,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
     ##  Gets all keys of settings in this container.
     #
     #   \return A set of all keys of settings in this container.
-    def getAllKeys(self):
+    def getAllKeys(self) -> List[str]:
         keys = set()
         for definition in self.definitions:
             keys |= definition.getAllKeys()
@@ -186,14 +212,42 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
 
         return json.dumps(data, separators = (", ", ": "), indent = 4) # Pretty print the JSON.
 
-    ##  \copydoc ContainerInterface::deserialize
-    #
-    #   Reimplemented from ContainerInterface
-    def deserialize(self, serialized):
+    def getConfigurationTypeFromSerialized(self, serialized: str) -> Optional[str]:
+        configuration_type = None
+        try:
+            parsed = self._readAndValidateSerialized(serialized)
+            configuration_type = parsed["metadata"]["type"]
+        except Exception as e:
+            Logger.log("d", "Could not get configuration type: %s", e)
+        return configuration_type
+
+    def _readAndValidateSerialized(self, serialized: str) -> dict:
         parsed = json.loads(serialized, object_pairs_hook=collections.OrderedDict)
 
         self._verifyJson(parsed)
 
+        parsed = self._preprocessParsedJson(parsed)
+
+        # If we do not have metadata or settings the file is invalid
+        if "metadata" not in parsed:
+            raise InvalidDefinitionError("Missing required metadata section")
+        if "version" not in parsed:
+            raise InvalidDefinitionError("Missing required version section")
+        if "settings" not in parsed:
+            raise InvalidDefinitionError("Missing required settings section")
+
+        return parsed
+
+    def getVersionFromSerialized(self, serialized: str) -> Optional[int]:
+        version = None
+        parsed = self._readAndValidateSerialized(serialized)
+        try:
+            version = int(parsed["version"])
+        except Exception as e:
+            Logger.log("d", "Could not get version from serialized: %s", e)
+        return version
+
+    def _preprocessParsedJson(self, parsed):
         # Pre-process the JSON data to include inherited data and overrides
         if "inherits" in parsed:
             inherited = self._resolveInheritance(parsed["inherits"])
@@ -208,12 +262,22 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
                     setting.update(value)
 
         # If we do not have metadata or settings the file is invalid
-        if not "metadata" in parsed:
+        if "metadata" not in parsed:
             raise InvalidDefinitionError("Missing required metadata section")
 
-        if not "settings" in parsed:
+        if "settings" not in parsed:
             raise InvalidDefinitionError("Missing required settings section")
 
+        return parsed
+
+    ##  \copydoc ContainerInterface::deserialize
+    #
+    #   Reimplemented from ContainerInterface
+    def deserialize(self, serialized):
+        # update the serialized data first
+        serialized = super().deserialize(serialized)
+        parsed = self._readAndValidateSerialized(serialized)
+
         # Update properties with the data from the JSON
         self._name = parsed["name"]
         self._metadata = parsed["metadata"]
@@ -229,7 +293,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
     ##  Find definitions matching certain criteria.
     #
     #   \param kwargs \type{dict} A dictionary of keyword arguments containing key-value pairs which should match properties of the definition.
-    def findDefinitions(self, **kwargs):
+    def findDefinitions(self, **kwargs) -> List[SettingDefinition]:
         if len(kwargs) == 1 and "key" in kwargs:
             # If we are searching for a single definition by exact key, we can speed up things by retrieving from the cache.
             key = kwargs.get("key")
@@ -242,10 +306,14 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
 
         return definitions
 
+    @classmethod
+    def getLoadingPriority(cls) -> int:
+        return 0
+
     # protected:
 
     # Load a file from disk, used to handle inheritance and includes
-    def _loadFile(self, file_name):
+    def _loadFile(self, file_name: str) -> dict:
         path = Resources.getPath(Resources.DefinitionContainers, file_name + ".def.json")
         contents = {}
         with open(path, encoding = "utf-8") as f:
@@ -255,31 +323,31 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
         return contents
 
     # Recursively resolve loading inherited files
-    def _resolveInheritance(self, file_name):
+    def _resolveInheritance(self, file_name: str) -> dict:
         result = {}
 
-        json = self._loadFile(file_name)
-        self._verifyJson(json)
+        json_dict = self._loadFile(file_name)
+        self._verifyJson(json_dict)
 
-        if "inherits" in json:
-            inherited = self._resolveInheritance(json["inherits"])
-            json = self._mergeDicts(inherited, json)
+        if "inherits" in json_dict:
+            inherited = self._resolveInheritance(json_dict["inherits"])
+            json_dict = self._mergeDicts(inherited, json_dict)
 
-        return json
+        return json_dict
 
     # Verify that a loaded json matches our basic expectations.
-    def _verifyJson(self, json):
-        if not "version" in json:
+    def _verifyJson(self, json_dict: dict):
+        if "version" not in json_dict:
             raise InvalidDefinitionError("Missing required property 'version'")
 
-        if not "name" in json:
+        if "name" not in json_dict:
             raise InvalidDefinitionError("Missing required property 'name'")
 
-        if json["version"] != self.Version:
-            raise IncorrectDefinitionVersionError("Definition uses version {0} but expected version {1}".format(json["version"], self.Version))
+        if json_dict["version"] != self.Version:
+            raise IncorrectDefinitionVersionError("Definition uses version {0} but expected version {1}".format(json_dict["version"], self.Version))
 
     # Recursively find a key in a dictionary
-    def _findInDict(self, dictionary, key):
+    def _findInDict(self, dictionary: dict, key: str):
         if key in dictionary: return dictionary[key]
         for k, v in dictionary.items():
             if isinstance(v, dict):
@@ -288,7 +356,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
                     return item
 
     # Recursively merge two dictionaries, returning a new dictionary
-    def _mergeDicts(self, first, second):
+    def _mergeDicts(self, first: dict, second: dict):
         result = copy.deepcopy(first)
         for key, value in second.items():
             if key in result:
@@ -302,7 +370,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
         return result
 
     # Recursively update relations of settings
-    def _updateRelations(self, definition):
+    def _updateRelations(self, definition: SettingDefinition):
         for property in SettingDefinition.getPropertyNames(DefinitionPropertyType.Function):
             self._processFunction(definition, property)
 
@@ -310,7 +378,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
             self._updateRelations(child)
 
     # Create relation objects for all settings used by a certain function
-    def _processFunction(self, definition, property):
+    def _processFunction(self, definition: SettingDefinition, property: str):
         try:
             function = getattr(definition, property)
         except AttributeError:
@@ -334,7 +402,7 @@ class DefinitionContainer(DefinitionContainerInterface, PluginObject):
             relation = SettingRelation(other, definition, RelationType.RequiredByTarget, property)
             other.relations.append(relation)
 
-    def _getDefinition(self, key):
+    def _getDefinition(self, key: str) -> SettingDefinition:
         definition = None
         if key in self._definition_cache:
             definition = self._definition_cache[key]
